@@ -18,9 +18,12 @@ import dataclasses
 import logging
 import pathlib
 import re
-from typing import Dict, Optional, TypedDict, Union
+import typing
+from typing import Dict, Literal, Optional, TypedDict, Union
 from urllib import parse as urlparse
 
+from libvcs.types import StrPath
+
 from .. import exc
 from .base import BaseProject, VCSLocation, convert_pip_url as base_convert_pip_url
 
@@ -51,7 +54,7 @@ class GitRemote:
 
 GitProjectRemoteDict = Dict[str, GitRemote]
 GitFullRemoteDict = Dict[str, GitRemoteDict]
-GitRemotesArgs = Union[None, GitFullRemoteDict, Dict[str, str]]
+GitRemotesArgs = Union[None, GitFullRemoteDict, GitProjectRemoteDict, Dict[str, str]]
 
 
 @dataclasses.dataclass
@@ -121,6 +124,9 @@ class GitStatus:
             re.VERBOSE | re.MULTILINE,
         )
         matches = pattern.search(value)
+
+        if matches is None:
+            raise Exception("Could not find match")
         return cls(**matches.groupdict())
 
 
@@ -152,9 +158,10 @@ def convert_pip_url(pip_url: str) -> VCSLocation:
 class GitProject(BaseProject):
     bin_name = "git"
     schemes = ("git", "git+http", "git+https", "git+ssh", "git+git", "git+file")
+    _remotes: GitProjectRemoteDict
 
     def __init__(
-        self, url: str, dir: str, remotes: GitRemotesArgs = None, *args, **kwargs
+        self, *, url: str, dir: StrPath, remotes: GitRemotesArgs = None, **kwargs
     ):
         """A git repository.
 
@@ -224,7 +231,11 @@ class GitProject(BaseProject):
                     )
                 elif isinstance(remote_url, dict):
                     self._remotes[remote_name] = GitRemote(
-                        **{**remote_url, "name": remote_name}
+                        **{
+                            "fetch_url": remote_url["fetch_url"],
+                            "push_url": remote_url["push_url"],
+                            "name": remote_name,
+                        }
                     )
                 elif isinstance(remote_url, GitRemote):
                     self._remotes[remote_name] = remote_url
@@ -235,19 +246,21 @@ class GitProject(BaseProject):
                 fetch_url=url,
                 push_url=url,
             )
-        super().__init__(url, dir, *args, **kwargs)
-        self.url = self.chomp_protocol(
-            (
-                self._remotes.get("origin")
-                if "origin" in self._remotes
-                else next(iter(self._remotes.items()))[1]
-            ).fetch_url
+        super().__init__(url=url, dir=dir, **kwargs)
+
+        origin = (
+            self._remotes.get("origin")
+            if "origin" in self._remotes
+            else next(iter(self._remotes.items()))[1]
         )
+        if origin is None:
+            raise Exception("Missing origin")
+        self.url = self.chomp_protocol(origin.fetch_url)
 
     @classmethod
-    def from_pip_url(cls, pip_url, *args, **kwargs):
+    def from_pip_url(cls, pip_url, **kwargs):
         url, rev = convert_pip_url(pip_url)
-        self = cls(url=url, rev=rev, *args, **kwargs)
+        self = cls(url=url, rev=rev, **kwargs)
 
         return self
 
@@ -374,6 +387,8 @@ class GitProject(BaseProject):
                 show_ref_output,
                 re.MULTILINE,
             )
+            if m is None:
+                raise exc.CommandError("Could not fetch remote names")
             git_remote_name = m.group("git_remote_name")
             git_tag = m.group("git_tag")
         self.log.debug("git_remote_name: %s" % git_remote_name)
@@ -480,7 +495,17 @@ class GitProject(BaseProject):
         cmd = ["submodule", "update", "--recursive", "--init"]
         self.run(cmd, log_in_real_time=True)
 
-    def remotes(self, flat=False) -> Dict:
+    @typing.overload
+    def remotes(self, flat: Literal[False] = ...):
+        ...
+
+    @typing.overload
+    def remotes(self, flat: Literal[True] = ...) -> GitFullRemoteDict:
+        ...
+
+    def remotes(
+        self, flat: bool = False
+    ) -> Union[GitProjectRemoteDict, GitFullRemoteDict]:
         """Return remotes like git remote -v.
 
         Parameters
@@ -495,15 +520,15 @@ class GitProject(BaseProject):
         remotes = {}
 
         cmd = self.run(["remote"])
-        ret = filter(None, cmd.split("\n"))
+        ret: filter[str] = filter(None, cmd.split("\n"))
 
         for remote_name in ret:
-            remotes[remote_name] = (
-                self.remote(remote_name) if flat else self.remote(remote_name).to_dict()
-            )
+            remote = self.remote(remote_name)
+            if remote is not None:
+                remotes[remote_name] = remote if flat else remote.to_dict()
         return remotes
 
-    def remote(self, name, **kwargs) -> GitRemote:
+    def remote(self, name, **kwargs) -> Optional[GitRemote]:
         """Get the fetch and push URL for a specified remote name.
 
         Parameters
